import {FC} from "react";
import {useNavigate} from "react-router-dom";
import {Menu, MenuProps, Space} from "antd";
import { useDoadActiveMenuValue, useDoadMenuItemsValue } from "../state";
import {DoadMenuItem} from "../types";

export const doadMenuWidth = 228

type MenuItem = Required<MenuProps>['items'][number];

function getMenuItem(items: DoadMenuItem[], key: string): DoadMenuItem | null {
  for (const item of items) {
    if (item.key === key) {
      return item
    }
    if (item.children != null) {
      const sub = getMenuItem(item.children, key)
      if (sub != null) {
        return sub
      }
    }
  }
  return null
}

function toRawMenuItems(items: DoadMenuItem[], deep = 0): MenuItem[] {
  return items.reduce((list, item, i) => {
    let label = item.title
    if (deep === 0 && item.icon != null) {
      label = (
        <Space style={{color: 'rgba(0,0,0,.88)', fontSize: '18px', width: '100%'}} size='middle'>
          <span style={{opacity: 0.6}}>{item.icon}</span>
          <span>{item.title}</span>
        </Space>
      )
    }
    let type: 'group' | undefined
    if (deep == 0) {
      type = 'group'
    }
    let children: MenuItem[] | undefined
    if (item.children?.length) {
      children = toRawMenuItems(item.children, deep + 1)
    } else if (deep == 0) {
      children = toRawMenuItems([item], deep + 1)
    }
    if (deep == 0 && i !== items.length - 1) {
      children ??= []
      children.push( {
        type: "divider",
        style: {
          width: '43%',
          margin: '12px 0 12px 50px',
        }
      })
    }

    list.push({
      key: item.key,
      label,
      type,
      children,
      style: {
        padding: type ? "0 18px" : "0 18px 0 46px",
        textAlign: "left",
      }
    })

    return list
  }, [] as MenuItem[])
}

export interface DoadMenuProps {
  onClick?: (item: DoadMenuItem, handled: boolean) => void
}

export const DoadMenu: FC<DoadMenuProps> = (props) => {
  const menuItems = useDoadMenuItemsValue()
  const activeMenuKey = useDoadActiveMenuValue()
  const navigate = useNavigate();

  const handleClick: MenuProps['onClick'] = (e) => {
    const item = getMenuItem(menuItems, e.key)
    let handled = true
    if (item?.handle != null) {
      item.handle(item, navigate)
    } else if (item?.path != null) {
      navigate(item.path)
    } else {
      handled = false
    }
    if (item != null) {
      props.onClick?.(item, handled)
    }
  }

  return (
    <Menu
      selectedKeys={[activeMenuKey].filter(Boolean) as string[]}
      style={{
        width: doadMenuWidth,
        margin: '40px 0',
        userSelect: 'none',
        backgroundColor: 'transparent',
      }}
      mode="inline"
      inlineIndent={0}
      items={toRawMenuItems(menuItems)}
      onClick={handleClick}
    />
  )
}

DoadMenu.displayName = 'DoadMenu'
